@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+import os.path
 import re
 
 import django
@@ -7,12 +8,15 @@ from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.forms.forms import BoundField
 from django.forms.models import formset_factory, modelformset_factory
-from django.template import Context, TemplateSyntaxError
-from django.template.loader import get_template_from_string
+from django.template import Context, TemplateSyntaxError, RequestContext
+from django.template import loader
 from django.middleware.csrf import _get_new_csrf_key
-from django.test import TestCase
+from django.shortcuts import render_to_response
+from django.test import TestCase, RequestFactory
+from django.test.utils import override_settings
 from django.utils.translation import ugettext_lazy as _
 
+from crispy_forms.compatibility import string_types, text_type
 from crispy_forms.exceptions import DynamicError
 from crispy_forms.helper import FormHelper, FormHelpersException
 from crispy_forms.layout import Submit, Reset, Hidden, Button
@@ -27,6 +31,7 @@ from crispy_forms.bootstrap import (
 )
 from crispy_forms.utils import render_crispy_form
 from crispy_forms.templatetags.crispy_forms_tags import CrispyFormNode
+from crispy_forms.templatetags.crispy_forms_field import crispy_addon
 
 from crispy_forms.tests.forms import (
     TestForm, TestForm2, TestForm3, ExampleForm, CheckboxesTestForm,
@@ -34,15 +39,36 @@ from crispy_forms.tests.forms import (
 )
 
 
-class TestBasicFunctionalityTags(TestCase):
+class CrispyTestCase(TestCase):
     def setUp(self):
-        pass
+        template_dirs = [os.path.join(os.path.dirname(__file__), 'templates')]
+        template_dirs = template_dirs + list(settings.TEMPLATE_DIRS)
+        template_loaders = ['django.template.loaders.filesystem.Loader']
+        template_loaders = template_loaders + list(settings.TEMPLATE_LOADERS)
+
+        # ensuring test templates directory is loaded first
+        self.__overriden_settings = override_settings(**{
+            'TEMPLATE_LOADERS': template_loaders,
+            'TEMPLATE_DIRS': template_dirs,
+        })
+        self.__overriden_settings.enable()
+
+        # resetting template loaders cache
+        self.__template_source_loaders = loader.template_source_loaders
+        loader.template_source_loaders = None
 
     def tearDown(self):
-        pass
+        loader.template_source_loaders = self.__template_source_loaders
+        self.__overriden_settings.disable()
 
+    @property
+    def current_template_pack(self):
+        return getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap')
+
+
+class TestBasicFunctionalityTags(CrispyTestCase):
     def test_as_crispy_errors_form_without_non_field_errors(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {{ form|as_crispy_errors }}
         """)
@@ -54,7 +80,7 @@ class TestBasicFunctionalityTags(TestCase):
         self.assertFalse("errorMsg" in html or "alert" in html)
 
     def test_as_crispy_errors_form_with_non_field_errors(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {{ form|as_crispy_errors }}
         """)
@@ -68,7 +94,7 @@ class TestBasicFunctionalityTags(TestCase):
         self.assertFalse("<h3>" in html)
 
     def test_crispy_filter_with_form(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {{ form|crispy }}
         """)
@@ -79,7 +105,7 @@ class TestBasicFunctionalityTags(TestCase):
         self.assertTrue("id_is_company" in html)
 
     def test_crispy_filter_with_formset(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {{ testFormset|crispy }}
         """)
@@ -97,7 +123,7 @@ class TestBasicFunctionalityTags(TestCase):
         self.assertTrue('form-MAX_NUM_FORMS' in html)
 
     def test_classes_filter(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_field %}
             {{ testField|classes }}
         """)
@@ -110,7 +136,7 @@ class TestBasicFunctionalityTags(TestCase):
 
     def test_crispy_field_and_class_converters(self):
         if hasattr(settings, 'CRISPY_CLASS_CONVERTERS'):
-            template = get_template_from_string(u"""
+            template = loader.get_template_from_string(u"""
                 {% load crispy_forms_field %}
                 {% crispy_field testField 'class' 'error' %}
             """)
@@ -123,14 +149,26 @@ class TestBasicFunctionalityTags(TestCase):
             self.assertTrue('error' in html)
             self.assertTrue('inputtext' in html)
 
+    def test_crispy_addon(self):
+        test_form = TestForm()
+        field_instance = test_form.fields['email']
+        bound_field = BoundField(test_form, field_instance, 'email')
+        # prepend tests
+        self.assertIn("input-prepend", crispy_addon(bound_field, prepend="Work"))
+        self.assertNotIn("input-append", crispy_addon(bound_field, prepend="Work"))
+        # append tests
+        self.assertNotIn("input-prepend", crispy_addon(bound_field, append="Primary"))
+        self.assertIn("input-append", crispy_addon(bound_field, append="Secondary"))
+        # prepend and append tests
+        self.assertIn("input-append", crispy_addon(bound_field, prepend="Work", append="Primary"))
+        self.assertIn("input-prepend", crispy_addon(bound_field, prepend="Work", append="Secondary"))
+        # errors
+        with self.assertRaises(TypeError):
+            crispy_addon()
+            crispy_addon(bound_field)
 
-class TestFormHelpers(TestCase):
+class TestFormHelpers(CrispyTestCase):
     urls = 'crispy_forms.tests.urls'
-    def setUp(self):
-        pass
-
-    def tearDown(self):
-        pass
 
     def test_inputs(self):
         form_helper = FormHelper()
@@ -139,7 +177,7 @@ class TestFormHelpers(TestCase):
         form_helper.add_input(Hidden('my-hidden', 'Hidden'))
         form_helper.add_input(Button('my-button', 'Button'))
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -152,7 +190,7 @@ class TestFormHelpers(TestCase):
         self.assertTrue('name="my-hidden"' in html)
         self.assertTrue('id="button-id-my-button"' in html)
 
-        if settings.CRISPY_TEMPLATE_PACK == 'uni_form':
+        if self.current_template_pack == 'uni_form':
             self.assertTrue('submit submitButton' in html)
             self.assertTrue('reset resetButton' in html)
             self.assertTrue('class="button"' in html)
@@ -178,7 +216,7 @@ class TestFormHelpers(TestCase):
         form_helper.form_action = 'simpleAction'
         form_helper.form_error_title = 'ERRORS'
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy testForm form_helper %}
         """)
@@ -196,7 +234,7 @@ class TestFormHelpers(TestCase):
         self.assertTrue('id="this-form-rocks"' in html)
         self.assertTrue('action="%s"' % reverse('simpleAction') in html)
 
-        if (settings.CRISPY_TEMPLATE_PACK == 'uni_form'):
+        if (self.current_template_pack == 'uni_form'):
             self.assertTrue('class="uniForm' in html)
 
         self.assertTrue("ERRORS" in html)
@@ -217,7 +255,7 @@ class TestFormHelpers(TestCase):
         form.helper.form_show_errors = True
         form.is_valid()
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy testForm %}
         """)
@@ -228,7 +266,7 @@ class TestFormHelpers(TestCase):
 
         # Ensure those errors were rendered
         self.assertTrue('<li>Passwords dont match</li>' in html)
-        self.assertTrue(unicode(_('This field is required.')) in html)
+        self.assertTrue(text_type(_('This field is required.')) in html)
         self.assertTrue('error' in html)
 
         # Now we render without errors
@@ -238,7 +276,7 @@ class TestFormHelpers(TestCase):
 
         # Ensure errors were not rendered
         self.assertFalse('<li>Passwords dont match</li>' in html)
-        self.assertFalse(unicode(_('This field is required.')) in html)
+        self.assertFalse(text_type(_('This field is required.')) in html)
         self.assertFalse('error' in html)
 
     def test_form_show_errors(self):
@@ -337,7 +375,7 @@ class TestFormHelpers(TestCase):
         html = render_crispy_form(form)
 
         # Check that help goes before error, otherwise CSS won't work
-        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+        if self.current_template_pack == 'bootstrap':
             help_position = html.find('<span id="hint_id_email" class="help-inline">')
             error_position = html.find('<p id="error_1_id_email" class="help-block">')
             self.assertTrue(help_position < error_position)
@@ -352,7 +390,7 @@ class TestFormHelpers(TestCase):
         html = render_crispy_form(form)
 
         # Check that error goes before help, otherwise CSS won't work
-        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+        if self.current_template_pack == 'bootstrap':
             error_position = html.find('<span id="error_1_id_email" class="help-inline">')
             help_position = html.find('<p id="hint_id_email" class="help-block">')
             self.assertTrue(error_position < help_position)
@@ -407,7 +445,7 @@ class TestFormHelpers(TestCase):
         self.assertEqual(helper['form_id'], 'test-form')
 
     def test_without_helper(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form %}
         """)
@@ -418,15 +456,15 @@ class TestFormHelpers(TestCase):
         self.assertTrue('<form' in html)
         self.assertTrue('method="post"' in html)
         self.assertFalse('action' in html)
-        if (settings.CRISPY_TEMPLATE_PACK == 'uni_form'):
+        if (self.current_template_pack == 'uni_form'):
             self.assertTrue('uniForm' in html)
 
     def test_template_pack_override(self):
-        current_pack = settings.CRISPY_TEMPLATE_PACK
+        current_pack = self.current_template_pack
         override_pack = current_pack == 'uni_form' and 'bootstrap' or 'uni_form'
 
         # Syntax {% crispy form 'template_pack_name' %}
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {%% load crispy_forms_tags %%}
             {%% crispy form "%s" %%}
         """ % override_pack)
@@ -434,7 +472,7 @@ class TestFormHelpers(TestCase):
         html = template.render(c)
 
         # Syntax {% crispy form helper 'template_pack_name' %}
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {%% load crispy_forms_tags %%}
             {%% crispy form form_helper "%s" %%}
         """ % override_pack)
@@ -449,7 +487,7 @@ class TestFormHelpers(TestCase):
             self.assertTrue('uniForm' in html2)
 
     def test_invalid_helper(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -464,7 +502,7 @@ class TestFormHelpers(TestCase):
         del settings.CRISPY_FAIL_SILENTLY
 
     def test_formset_with_helper_without_layout(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy testFormSet formset_helper %}
         """)
@@ -493,12 +531,12 @@ class TestFormHelpers(TestCase):
         self.assertTrue('method="post"' in html)
         self.assertTrue('id="thisFormsetRocks"' in html)
         self.assertTrue('action="%s"' % reverse('simpleAction') in html)
-        if (settings.CRISPY_TEMPLATE_PACK == 'uni_form'):
+        if (self.current_template_pack == 'uni_form'):
             self.assertTrue('class="uniForm' in html)
 
     def test_CSRF_token_POST_form(self):
         form_helper = FormHelper()
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -514,7 +552,7 @@ class TestFormHelpers(TestCase):
     def test_CSRF_token_GET_form(self):
         form_helper = FormHelper()
         form_helper.form_method = 'GET'
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -524,6 +562,13 @@ class TestFormHelpers(TestCase):
 
         self.assertFalse("<input type='hidden' name='csrfmiddlewaretoken'" in html)
 
+    def test_disable_csrf(self):
+        form = TestForm()
+        helper = FormHelper()
+        helper.disable_csrf = True
+        html = render_crispy_form(form, helper, {'csrf_token': _get_new_csrf_key()})
+        self.assertFalse('csrf' in html)
+
     def test_render_hidden_fields(self):
         test_form = TestForm()
         test_form.helper = FormHelper()
@@ -561,8 +606,28 @@ class TestFormHelpers(TestCase):
         html = render_crispy_form(test_form)
         self.assertEqual(html.count('<input'), 7)
 
+    def test_helper_custom_template(self):
+        form = TestForm()
+        form.helper = FormHelper()
+        form.helper.template = 'custom_form_template.html'
 
-class TestFormLayout(TestCase):
+        html = render_crispy_form(form)
+        self.assertTrue("<h1>Special custom form</h1>" in html)
+
+    def test_helper_custom_field_template(self):
+        form = TestForm()
+        form.helper = FormHelper()
+        form.helper.layout = Layout(
+            'password1',
+            'password2',
+        )
+        form.helper.field_template = 'custom_field_template.html'
+
+        html = render_crispy_form(form)
+        self.assertEqual(html.count("<h1>Special custom field</h1>"), 2)
+
+
+class TestFormLayout(CrispyTestCase):
     urls = 'crispy_forms.tests.urls'
 
     def test_invalid_unicode_characters(self):
@@ -574,7 +639,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -593,7 +658,7 @@ class TestFormLayout(TestCase):
             'first_name',
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -609,7 +674,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -626,7 +691,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -639,7 +704,7 @@ class TestFormLayout(TestCase):
         form = ExampleForm()
         form2 = TestForm()
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {{ form.as_ul }}
             {% crispy form2 %}
@@ -696,7 +761,7 @@ class TestFormLayout(TestCase):
         self.assertEqual(html.count('name="whatever"'), 1)
         self.assertEqual(html.count('value="something"'), 1)
 
-        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+        if self.current_template_pack == 'bootstrap':
             # Make sure white spaces between buttons are there in bootstrap
             self.assertEqual(len(re.findall(r'</button> <', html)), 3)
 
@@ -731,7 +796,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -749,7 +814,7 @@ class TestFormLayout(TestCase):
         self.assertTrue('test-fieldset="123"' in html)
         self.assertTrue('id="row_passwords"' in html)
 
-        if settings.CRISPY_TEMPLATE_PACK == 'uni_form':
+        if self.current_template_pack == 'uni_form':
             self.assertTrue('class="formRow rows"' in html)
         else:
             self.assertTrue('class="row rows"' in html)
@@ -787,7 +852,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -843,7 +908,7 @@ class TestFormLayout(TestCase):
             )
         )
 
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -863,7 +928,7 @@ class TestFormLayout(TestCase):
         self.assertFalse('last_name' in html)
 
     def test_change_layout_dynamically_delete_field(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form_helper %}
         """)
@@ -931,11 +996,11 @@ class TestFormLayout(TestCase):
             self.assertEqual(html.count(
                 'type="hidden" name="form-INITIAL_FORMS" value="0" id="id_form-INITIAL_FORMS"'
             ), 1)
-            if django_version < '1.4.4':
+            if (django_version >= '1.4' and django_version < '1.4.4') or django_version < '1.3.6':
                 self.assertEqual(html.count(
                     'type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS"'
                 ), 1)
-            if django_version >= '1.4.4' and django_version < 1.5:
+            else:
                 self.assertEqual(html.count(
                     'type="hidden" name="form-MAX_NUM_FORMS" value="1000" id="id_form-MAX_NUM_FORMS"'
                 ), 1)
@@ -964,7 +1029,7 @@ class TestFormLayout(TestCase):
         self.assertTrue('Item 2' in html)
         self.assertTrue('Item 3' in html)
         self.assertEqual(html.count('Note for first form only'), 1)
-        if settings.CRISPY_TEMPLATE_PACK == 'uni_form':
+        if self.current_template_pack == 'uni_form':
             self.assertEqual(html.count('formRow'), 3)
         else:
             self.assertEqual(html.count('row'), 3)
@@ -990,11 +1055,11 @@ class TestFormLayout(TestCase):
             self.assertEqual(html.count(
                 'type="hidden" name="form-INITIAL_FORMS" value="0" id="id_form-INITIAL_FORMS"'
             ), 1)
-            if django_version < '1.4.4':
+            if (django_version >= '1.4' and django_version < '1.4.4') or django_version < '1.3.6':
                 self.assertEqual(html.count(
                     'type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS"'
                 ), 1)
-            if django_version >= '1.4.4' and django_version < 1.5:
+            else:
                 self.assertEqual(html.count(
                     'type="hidden" name="form-MAX_NUM_FORMS" value="1000" id="id_form-MAX_NUM_FORMS"'
                 ), 1)
@@ -1016,7 +1081,7 @@ class TestFormLayout(TestCase):
         self.assertEqual(html.count('password'), 0)
 
     def test_multiwidget_field(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form %}
         """)
@@ -1043,7 +1108,7 @@ class TestFormLayout(TestCase):
         self.assertEqual(html.count('type="hidden"'), 1)
 
     def test_i18n(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy form form.helper %}
         """)
@@ -1094,10 +1159,28 @@ class TestFormLayout(TestCase):
         self.assertEqual(html.count('checkbox inline"'), 3)
         self.assertEqual(html.count('inline"'), 3)
 
+    def test_keepcontext_context_manager(self):
+        # Test case for issue #180
+        # Apparently it only manifest when using render_to_response this exact way
+        form = CheckboxesTestForm()
+        form.helper = FormHelper()
+        # We use here InlineCheckboxes as it updates context in an unsafe way
+        form.helper.layout = Layout(
+            'checkboxes',
+            InlineCheckboxes('alphacheckboxes'),
+            'numeric_multiple_checkboxes'
+        )
+        request_factory = RequestFactory()
+        request = request_factory.get('/')
+        context = RequestContext(request, {'form': form})
 
-class TestLayoutObjects(TestCase):
+        response = render_to_response('crispy_render_template.html', context)
+        self.assertEqual(response.content.count(b'checkbox inline'), 3)
+
+
+class TestLayoutObjects(CrispyTestCase):
     def test_field_type_hidden(self):
-        template = get_template_from_string(u"""
+        template = loader.get_template_from_string(u"""
             {% load crispy_forms_tags %}
             {% crispy test_form %}
         """)
@@ -1122,9 +1205,28 @@ class TestLayoutObjects(TestCase):
 
     def test_field_wrapper_class(self):
         html = Field('email', wrapper_class="testing").render(TestForm(), "", Context())
-        if settings.CRISPY_TEMPLATE_PACK == 'bootstrap':
+        if self.current_template_pack == 'bootstrap':
             self.assertEqual(html.count('class="control-group testing"'), 1)
 
+    def test_custom_django_widget(self):
+        class CustomRadioSelect(forms.RadioSelect):
+            pass
+
+        class CustomCheckboxSelectMultiple(forms.CheckboxSelectMultiple):
+            pass
+
+        if self.current_template_pack == 'bootstrap':
+            # Make sure an inherited RadioSelect gets rendered as it
+            form = CheckboxesTestForm()
+            form.fields['inline_radios'].widget = CustomRadioSelect()
+            html = Field('inline_radios').render(form, "", Context())
+            self.assertTrue('class="radio"' in html)
+
+            # Make sure an inherited CheckboxSelectMultiple gets rendered as it
+            form.fields['checkboxes'].widget = CustomCheckboxSelectMultiple()
+            html = Field('checkboxes').render(form, "", Context())
+            self.assertTrue('class="checkbox"' in html)
+
     def test_appended_prepended_text(self):
         test_form = TestForm()
         test_form.helper = FormHelper()
@@ -1246,14 +1348,16 @@ class TestLayoutObjects(TestCase):
         )
         html = render_crispy_form(test_form)
 
-        if settings.CRISPY_TEMPLATE_PACK == 'uni_form':
+        if self.current_template_pack == 'uni_form':
             self.assertEqual(html.count('\n'), 22)
         else:
             self.assertEqual(html.count('\n'), 24)
 
 
-class TestDynamicLayouts(TestCase):
+class TestDynamicLayouts(CrispyTestCase):
     def setUp(self):
+        super(TestDynamicLayouts, self).setUp()
+
         self.advanced_layout = Layout(
             Div(
                 Div(Div('email')),
@@ -1358,6 +1462,35 @@ class TestDynamicLayouts(TestCase):
         self.assertEqual(layout.fields[1], 'password1')
         self.assertEqual(layout.fields[2], 'password2')
 
+    def test_wrap_together_partial_slices(self):
+        helper = FormHelper()
+        layout = Layout(
+            'email',
+            'password1',
+            'password2',
+        )
+        helper.layout = layout
+
+        helper[:2].wrap_together(Field, css_class="test-class")
+        self.assertTrue(isinstance(layout.fields[0], Field))
+        self.assertEqual(layout.fields[1], 'password2')
+        self.assertEqual(layout.fields[0][0], 'email')
+        self.assertEqual(layout.fields[0][1], 'password1')
+
+        helper = FormHelper()
+        layout = Layout(
+            'email',
+            'password1',
+            'password2',
+        )
+        helper.layout = layout
+
+        helper[1:].wrap_together(Field, css_class="test-class")
+        self.assertEqual(layout.fields[0], 'email')
+        self.assertTrue(isinstance(layout.fields[1], Field))
+        self.assertEqual(layout.fields[1][0], 'password1')
+        self.assertEqual(layout.fields[1][1], 'password2')
+
     def test_update_attributes_and_wrap_once(self):
         helper = FormHelper()
         layout = Layout(
@@ -1387,12 +1520,12 @@ class TestDynamicLayouts(TestCase):
         )
         helper.layout = layout
 
-        helper.filter(basestring, greedy=True).wrap_once(Field)
+        helper.filter(string_types, greedy=True).wrap_once(Field)
         helper.filter(Field, greedy=True).update_attributes(readonly=True)
 
         self.assertTrue(isinstance(layout[0], Field))
         self.assertTrue(isinstance(layout[1][0], Field))
-        self.assertTrue(isinstance(layout[1][0][0], basestring))
+        self.assertTrue(isinstance(layout[1][0][0], string_types))
         self.assertTrue(isinstance(layout[2], Field))
         self.assertEqual(layout[1][0].attrs, {'readonly': True})
         self.assertEqual(layout[0].attrs, {'readonly': True})
@@ -1435,7 +1568,7 @@ class TestDynamicLayouts(TestCase):
             Div('password1'),
             'password2',
         )
-        self.assertEqual(layout_3.get_layout_objects(basestring, max_level=2), [
+        self.assertEqual(layout_3.get_layout_objects(string_types, max_level=2), [
             [[0], 'email'],
             [[1, 0], 'password1'],
             [[2], 'password2']
@@ -1491,7 +1624,7 @@ class TestDynamicLayouts(TestCase):
         )
         helper.layout = layout
 
-        helper.filter(basestring).wrap(Field, css_class="test-class")
+        helper.filter(string_types).wrap(Field, css_class="test-class")
         self.assertTrue(isinstance(layout.fields[0], Field))
         self.assertTrue(isinstance(layout.fields[1], Div))
         self.assertTrue(isinstance(layout.fields[2], Field))
@@ -1632,8 +1765,8 @@ class TestDynamicLayouts(TestCase):
         self.assertTrue(isinstance(form.helper.layout[1], Field))
         # Check others stay the same
         self.assertTrue(isinstance(form.helper.layout[0][3][1], HTML))
-        self.assertTrue(isinstance(form.helper.layout[0][1][0][0], basestring))
-        self.assertTrue(isinstance(form.helper.layout[0][4][0], basestring))
+        self.assertTrue(isinstance(form.helper.layout[0][1][0][0], string_types))
+        self.assertTrue(isinstance(form.helper.layout[0][4][0], string_types))
 
     def test_all_without_layout(self):
         form = TestForm()
@@ -1706,8 +1839,8 @@ class TestDynamicLayouts(TestCase):
         self.assertTrue(isinstance(layout[0][0], Div))
         self.assertTrue(isinstance(layout[0][0][0], Div))
         self.assertTrue(isinstance(layout[0][1], Div))
-        self.assertTrue(isinstance(layout[0][1][0], basestring))
-        self.assertTrue(isinstance(layout[0][2], basestring))
+        self.assertTrue(isinstance(layout[0][1][0], string_types))
+        self.assertTrue(isinstance(layout[0][2], string_types))
 
     def test__getattr__append_layout_object(self):
         layout = Layout(
@@ -1715,8 +1848,8 @@ class TestDynamicLayouts(TestCase):
         )
         layout.append('password1')
         self.assertTrue(isinstance(layout[0], Div))
-        self.assertTrue(isinstance(layout[0][0], basestring))
-        self.assertTrue(isinstance(layout[1], basestring))
+        self.assertTrue(isinstance(layout[0][0], string_types))
+        self.assertTrue(isinstance(layout[1], string_types))
 
     def test__setitem__layout_object(self):
         layout = Layout(
